package com.cch_lab.android.apply.prototype.tinybasic.command;

import com.cch_lab.android.apply.prototype.tinybasic.command.item.AssignmentCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.AssignmentWithDefineCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.BinaryOperatorCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.BlankBracketCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.BlankParenthesesCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.BuiltInFunctionCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.DefineIntegerVariableCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.DefineStringVariableCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.ExpressionCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.InputFunctionCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.KeywordCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.LeftParenthesesBinaryOperationCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.NoneCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.OperationBothParenthesesCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.OperationLeftParenthesesCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.OperationRightParenthesesCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.OperationValueCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.PrintFunctionCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.RightParenthesesBinaryOperationCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.UnaryOperatorCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.item.VariableCommand;


/**
 * コマンド・マネジャ・クラス.
 * {@inheritDoc}
 */
public class CommandManager extends AbstractCommandManager {

    /**
     * デフォルト・コンストラクタ.
     */
    public CommandManager() {
    }

    @Override
    public void setup() {
        // 何もしない
        register(NoneCommand.class);
        /*
        Program.class,
        Statement.class,
        Statements.class,

        AssignmentStatement.class,
        AssignmentWithDefineStatement.class,
        ExpressionStatement.class,

        StringKeyword.class,
        IntegerKeyword.class,
        InputKeyword.class,
        PrintKeyword.class,

        LineComment.class,
        BlockComment.class,
        BeginBlockComment.class,
        MiddleBlockComment.class,
        EndBlockComment.class
        */

        // キーワード名称を返す
        register(KeywordCommand.class);
        /*
        StringKeyword.class,
        IntegerKeyword.class,
        InputKeyword.class,
        PrintKeyword.class
        */

        // 変数名を返す
        register(VariableCommand.class);
        /*
        Variable.class
        */

        // 変数定義を行う
        register(DefineStringVariableCommand.class);
        register(DefineIntegerVariableCommand.class);
        /*
        DefineStringVariable.class,
        DefineIntegerVariable.class,
        */

        // 変数への代入を行う
        register(AssignmentWithDefineCommand.class);
        register(AssignmentCommand.class);
        /*
        AssignmentWithDefine.class,
        Assignment.class,
        */

        // 演算値を返す
        register(OperationValueCommand.class);
        /*
        OperationValue.class,
        */

        // 単項演算の評価結果を返す
        register(UnaryOperatorCommand.class);
        /*
        UnaryOperator.class
        */

        // 二項演算の評価結果を返す
        register(BinaryOperatorCommand.class);
        /*
        PrimaryBinaryOperator.class,
        SecondaryBinaryOperator.class
        */

        // 空括弧を処理する
        register(BlankParenthesesCommand.class);
        register(BlankBracketCommand.class);
        /*
        BlankParentheses.class,
        BlankBracket.class,
        */

        // 左右括弧付きの二項演算をマークする
        register(RightParenthesesBinaryOperationCommand.class);
        register(LeftParenthesesBinaryOperationCommand.class);
        /*
        RightParenthesesBinaryOperator.class
        LeftParenthesesBinaryOperator.class
        */

        // 括弧付きの二項演算評価結果を返す
        register(OperationLeftParenthesesCommand.class);
        register(OperationRightParenthesesCommand.class);
        register(OperationBothParenthesesCommand.class);
        /*
        OperationLeftParentheses.class,
        OperationRightParentheses.class,
        OperationBothParentheses.class,
        */

        // 組み込み関数を実行する
        register(InputFunctionCommand.class);
        register(PrintFunctionCommand.class);
        register(BuiltInFunctionCommand.class);
        /*
        InputFunction.class,
        PrintFunction.class,
        BuiltInFunction.class
        */

        // 式の評価結果を返す
        register(ExpressionCommand.class);
        /*
        Expression.class,
        */
    }
}
