package com.cch_lab.android.apply.prototype.tinybasic.command.item;

import com.cch_lab.android.apply.prototype.tinybasic.command.AbstractCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.CommandRuntimeEnvironment;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.AbstractSyntax;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.SyntaxTree;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.InputFunction;

import java.io.IOException;


/**
 * (組込関数)input・コマンド.
 * <p>
 *     (組込関数)inputを表すコマンドです。
 * </p>
 */
public class InputFunctionCommand extends AbstractCommand {

    /**
     *{@inheritDoc}
     */
    @Override
    public Class<? extends AbstractSyntax>[] getSupportSyntax() {
        Class[] patterns =
                    new Class[]{
                        InputFunction.class,
                    };
        return (Class<? extends AbstractSyntax>[]) patterns;
    }

    /**
     *{@inheritDoc}
     */
    @Override
    public Object evaluate(
            CommandRuntimeEnvironment runtime,
            Object leftLeafResult, Object middleLeafResult, Object rightLeafResult) {

        // 対応構文ノード取得
        SyntaxTree node = getParameterSyntaxTree();

        // 関数名取得 (関数名/キーワード名を取得)
        String functionName = node.getMiddleLeaf().getMiddleToken().getTokenSource().getSource();

        // 関数名に対応するシステム機能を実行
        if("input".equals(functionName)) {
            StringBuilder sb = new StringBuilder();
            int length = 0;
            do {
                byte[] buffer = new byte[512];
                try {
                    length = System.in.read(buffer);
                } catch(IOException e) {
                    e.printStackTrace();
                }
                sb.append(new String(buffer));
            } while(length != -1);
            return sb.toString();
        }
        return null;
    }
}
