package com.cch_lab.android.apply.prototype.tinybasic.command.item;

import com.cch_lab.android.apply.prototype.tinybasic.command.AbstractCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.CommandRuntimeEnvironment;
import com.cch_lab.android.apply.prototype.tinybasic.command.Void;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.AbstractSyntax;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.SyntaxTree;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.Expression;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.OperationValue;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.PrintFunction;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.UnaryOperator;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.Variable;
import com.cch_lab.android.apply.prototype.tinybasic.token.AbstractToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.IntegerNumberLiteralToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.StringLiteralToken;


/**
 * 演算値・コマンド.
 * <p>
 *     演算値を表すコマンドです。
 * </p>
 */
public class OperationValueCommand extends AbstractCommand {

    /**
     *{@inheritDoc}
     */
    @Override
    public Class<? extends AbstractSyntax>[] getSupportSyntax() {
        Class[] patterns =
                    new Class[]{
                        OperationValue.class,
                    };
        return (Class<? extends AbstractSyntax>[]) patterns;
    }

    /**
     *{@inheritDoc}
     */
    @Override
    public Object evaluate(
            CommandRuntimeEnvironment runtime,
            Object leftLeafResult, Object middleLeafResult, Object rightLeafResult) {

        // 対応構文ノードとトークン＆構文取得
        SyntaxTree node = getParameterSyntaxTree();

        if (node.getMiddleToken() != null) {
            AbstractToken token = node.getMiddleToken();

            // リテラルの場合
            if (token.getClass().equals(IntegerNumberLiteralToken.class)) {
                return Integer.valueOf(token.getTokenSource().getSource());
            } else
            if (token.getClass().equals(StringLiteralToken.class)) {
                return OperationValueCommand.stripDoubleQuote(token.getTokenSource().getSource());
            }
        } else
        if (node.getMiddleLeaf() != null) {
            AbstractSyntax syntax = node.getMiddleLeaf().getNodeSyntax();

            if (syntax.getClass().equals(Variable.class)) {
                // 変数の場合
                String name = (String)middleLeafResult;
                AbstractToken token = syntax.getBodySyntax().getToken();
                Class<?> type = runtime.getVariableType(name, token);
                return runtime.getVariableValue(name, type, token);
            } else {
                // 式が演算(値を返す)の場合
                return middleLeafResult;
            }
        }
        return new Void();
    }

    public static String stripDoubleQuote(String source) {
        return source.substring(1, source.length()-1);
    }

}
