package com.cch_lab.android.apply.prototype.tinybasic.command.item;

import com.cch_lab.android.apply.prototype.tinybasic.command.CommandRuntimeException;
import com.cch_lab.android.apply.prototype.tinybasic.command.Void;
import com.cch_lab.android.apply.prototype.tinybasic.command.AbstractCommand;
import com.cch_lab.android.apply.prototype.tinybasic.command.CommandRuntimeEnvironment;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.AbstractSyntax;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.SyntaxTree;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.OperationValue;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.UnaryOperator;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.Variable;
import com.cch_lab.android.apply.prototype.tinybasic.token.AbstractToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.IntegerNumberLiteralToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.MinusOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.NegativeOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.PlusOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.StringLiteralToken;


/**
 * 単項演算・コマンド.
 * <p>
 *     単項演算を表すコマンドです。
 * </p>
 */
public class UnaryOperatorCommand extends AbstractCommand {

    /**
     *{@inheritDoc}
     */
    @Override
    public Class<? extends AbstractSyntax>[] getSupportSyntax() {
        Class[] patterns =
                    new Class[]{
                            UnaryOperator.class
                    };
        return (Class<? extends AbstractSyntax>[]) patterns;
    }

    /**
     *{@inheritDoc}
     */
    @Override
    public Object evaluate(
            CommandRuntimeEnvironment runtime,
            Object leftLeafResult, Object middleLeafResult, Object rightLeafResult) {

        // 対応構文ノードとトークン取得
        SyntaxTree node = getParameterSyntaxTree();
        AbstractToken token = node.getMiddleToken();

        // 値補正
        AbstractToken valueToken = node.getRightToken();
        if (valueToken != null) {
            // トークン値取得
            Class<?> valueClass = valueToken.getClass();
            rightLeafResult = valueClass.equals(IntegerNumberLiteralToken.class)
                    ? Integer.valueOf(valueToken.getTokenSource().getSource())
                    : valueClass.equals(StringLiteralToken.class)
                    ? OperationValueCommand.stripDoubleQuote(valueToken.getTokenSource().getSource())
                    : new Void();
        } else
        if (rightLeafResult != null) {
            Class<?> valueClass = node.getRightLeaf().getNodeSyntax().getClass();
            if (valueClass.equals(Variable.class)) {
                // 変数値取得
                String name = (String)rightLeafResult;
                Class<?> type = runtime.getVariableType(name, token);
                rightLeafResult = runtime.getVariableValue(name, type, token);
            }
        }

        // 有効値チェック
        if (rightLeafResult == null) {
            throw new CommandRuntimeException(
                    "NullPointerException: " + runtime.getErrorInfo(token));
        } else
        if(rightLeafResult instanceof Void) {
            throw new CommandRuntimeException(
                    "The return value is a void: " + runtime.getErrorInfo(token));
        }

        // 値型取得
        Class<?> type = rightLeafResult.getClass();

        // 単項演算値返却
        if(token.getClass().equals(PlusOperatorToken.class)) {
            if (type.equals(Integer.class)) {
                return rightLeafResult;
            }
        } else
        if(token.getClass().equals(MinusOperatorToken.class)) {
            if (type.equals(Integer.class)) {
                return -1 *((Integer)rightLeafResult);
            }
        } else
        if(token.getClass().equals(NegativeOperatorToken.class)) {
            if (type.equals(Boolean.class)) {
                return !((Boolean)rightLeafResult);
            }
        } else{
            throw new CommandRuntimeException("’" + type.getSimpleName() + "’ is a unknown type. " + runtime.getErrorInfo(token));
        }
        return new Void();
    }
}
