package com.cch_lab.android.apply.prototype.tinybasic.source;

import java.util.List;


/**
 * ソース・ストリーム.
 * <p>
 *     ソースコード行のストリームを表すクラスです。
 * </p>
 */
public class SourceStream {
    private int scanRowIndex;
    private int scanColumnIndex;
    private List<String> sourceLines;

    /**
     * コンストラクタ.
     * @param sourceLines ソースコード行リスト
     */
    public SourceStream(List<String> sourceLines) {
        this.sourceLines = sourceLines;
    }

    /**
     * 巻き戻し.
     */
    public void rewind() {
        scanRowIndex = 0;
        scanColumnIndex = 0;
    }

    /**
     * スキャン・ソースコード保持チェック.
     * @return スキャン・ソースコード保持結果 （true:スキャン対象あり、false:スキャン対象なし）
     */
    public boolean hasNextScanSource() {
        boolean hasNext = getScanSource(scanRowIndex, scanColumnIndex).trim().length() != 0;
        if (!hasNext && (sourceLines.size() > (scanRowIndex +1))) {
            // 次のソース行に進める場合
            scanRowIndex++;
            scanColumnIndex = 0;
            hasNext = true;
        }
        return hasNext;
    }

    /**
     * 次のスキャン・ソースコード位置に進む.
     * @param checkedSource チェック済のソースコード部
     */
    public void nextScanSource(String checkedSource) {
        scanColumnIndex += getScanSource().indexOf(checkedSource) + checkedSource.length();
    }

    /**
     * スキャン・ソースコード取得.
     * @param row ソースコード行インデックス
     * @param start ソースコード・スキャン開始位置インデックス
     * @return スキャン・ソースコード
     */
    String getScanSource(int row, int start) {
        return sourceLines.get(row).substring(start);
    }

    /**
     * スキャン・ソースコード取得.
     * @return スキャン・ソースコード
     */
    public String getScanSource() {
        return getScanSource(scanRowIndex, scanColumnIndex);
    }

    /**
     * (カレント)スキャン行インデックス取得.
     * @return ソースコード行インデックス
     */
    public int getScanRow() {
        return scanRowIndex;
    }

    /**
     * (カレント）スキャン開始桁インデックス取得.
     * @return ソースコード・スキャン開始桁インデックス
     */
    public int getScanColmn() {
        return scanColumnIndex;
    }

    /**
     * ソース取得.
     * @param row   ソースコード行インデックス
     * @param start ソースコード・開始位置インデックス
     * @param end   ソースコード・終了位置インデックス
     * @return ソース
     */
    public String getSource(int row, int start, int end) {
        return sourceLines.get(row).substring(start, end);
    }

    /**
     * ソース行取得.
     * @param row   ソースコード行インデックス
     * @return ソース行
     */
    public String getSourceLine(int row) {
        return sourceLines.get(row);
    }

    /**
     * ソース行個数取得.
     * @return ソース行個数
     */
    public int size() {
        return sourceLines.size();
    }
}
