package com.cch_lab.android.apply.prototype.tinybasic.syntax;

import com.cch_lab.android.apply.prototype.tinybasic.token.AbstractToken;

/**
 * 構文・ホルダ.
 * <p>
 *     トークン・ストリームから構文木への変換作業において、
 *     構文とトークンを透過的に扱うためのホルダ・クラスです。
 * </p>
 */
public class SyntaxHolder {
    /**
     * トークン｜構文・型列挙子.
     */
    public enum Type {
        /** 対象なし． */ NONE,
        /** トークン. */ TOKEN,
        /** 構文．   */ SYNTAX
    }

    private final Type           type;
    private final AbstractToken  token;
    private final AbstractSyntax syntax;

    /**
     * (保持対象なし)コンストラクタ.
     */
    public SyntaxHolder() {
        this.type   = Type.NONE;
        this.token  = null;
        this.syntax = null;
    }

    /**
     * （トークン保持）コンストラクタ.
     * @param abstractToken トークン
     */
    public SyntaxHolder(AbstractToken abstractToken) {
        this.type   = Type.TOKEN;
        this.token  = abstractToken;
        this.syntax = null;
    }

    /**
     * (構文保持)コンストラクタ.
     * @param syntax 構文
     */
    public SyntaxHolder(AbstractSyntax syntax) {
        this.type   = Type.SYNTAX;
        this.token  = null;
        this.syntax = syntax;
    }

    /**
     * トークン|構文・型取得.
     * @return 型 （トークン、構文、あるいは対象なし）
     */
    public Type getType() {
        return type;
    }

    /**
     * トークン|構文・型クラス取得.
     * @return トークン|構文・型クラス （対象なしの場合は null）
     */
    public Class<?> getTypeClass() {
        switch(type) {
            case TOKEN: {
                return token.getClass();
            }
            case SYNTAX: {
                return syntax.getClass();
            }
        }
        return null;
    }

    /**
     * トークン取得.
     * @return トークン (トークンを保持しない場合は、ｎｕｌｌが返る)
     */
    public AbstractToken getToken() {
        return token;
    }

    /**
     * 構文取得.
     * @return 構文 (構文を保持しない場合は、ｎｕｌｌが返る)
     */
    public AbstractSyntax getSyntax() {
        return syntax;
    }
}
