package com.cch_lab.android.apply.prototype.tinybasic.syntax;

import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.Assignment;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.AssignmentStatement;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.AssignmentWithDefine;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.AssignmentWithDefineStatement;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.BeginBlockComment;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.BlankBracket;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.BlankParentheses;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.BlockComment;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.BuiltInFunction;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.InputFunction;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.LeftParenthesesBinaryOperator;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.OperationBothParentheses;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.OperationLeftParentheses;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.OperationRightParentheses;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.OperationValue;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.PrintFunction;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.DefineIntegerVariable;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.DefineStringVariable;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.EndBlockComment;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.Expression;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.ExpressionStatement;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.PrimaryBinaryOperator;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.RightParenthesesBinaryOperator;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.UnaryOperator;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.InputKeyword;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.IntegerKeyword;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.LineComment;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.MiddleBlockComment;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.PrintKeyword;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.Program;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.SecondaryBinaryOperator;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.Statement;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.Statements;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.StringKeyword;
import com.cch_lab.android.apply.prototype.tinybasic.syntax.item.Variable;


/**
 * 構文・マネジャ・クラス.
 * {@inheritDoc}
 */
public class SyntaxManager extends AbstractSyntaxManager {

    /**
     * デフォルト・コンストラクタ.
     */
    public SyntaxManager() {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setup() {
        // コメント
        register(BlockComment.class);
        register(LineComment.class);
        register(BeginBlockComment.class);
        register(MiddleBlockComment.class);
        register(EndBlockComment.class);

        // キーワード (キーワードに当てはまる名称)
        register(PrintKeyword.class);
        register(InputKeyword.class);
        register(IntegerKeyword.class);
        register(StringKeyword.class);

        // 変数 (キーワードに当てはまらなかった名称)
        register(Variable.class);

        // 変数型宣言
        register(DefineIntegerVariable.class);
        register(DefineStringVariable.class);

        // 単項演算
        register(UnaryOperator.class);

        // 演算値
        register(OperationValue.class);

        // 左右丸括弧二項演算
        register(LeftParenthesesBinaryOperator.class);
        register(RightParenthesesBinaryOperator.class);

        // 二項演算
        register(PrimaryBinaryOperator.class);
        register(SecondaryBinaryOperator.class);

        // 空丸括弧
        register(BlankParentheses.class);

        // 空角括弧
        register(BlankBracket.class);

        // 演算丸括弧
        register(OperationLeftParentheses.class);
        register(OperationRightParentheses.class);
        register(OperationBothParentheses.class);

        // 組込関数 (特定)
        register(PrintFunction.class);
        register(InputFunction.class);

        // 組込関数 (一般)
        register(BuiltInFunction.class);

        // 二項演算 (二度目の正規化)
        register(PrimaryBinaryOperator.class);
        register(SecondaryBinaryOperator.class);

        // 演算式
        register(Expression.class);

        // 宣言付代入
        register(AssignmentWithDefine.class);

        // 代入
        register(Assignment.class);

        // 一般文
        register(AssignmentWithDefineStatement.class); //宣言付代入文
        register(AssignmentStatement.class);           // 代入文
        register(ExpressionStatement.class);           // 演算文

        // 文
        register(Statements.class);
        register(Statement.class);

        // プログラム
        register(Program.class);
    }
}
