package com.cch_lab.android.apply.prototype.tinybasic.syntax;


import com.cch_lab.android.apply.prototype.tinybasic.token.AbstractToken;

/**
 * 構文・パターン.
 * <p>
 *     構文パターン情報を保管するクラスです。
 * </p>
 */
public class SyntaxPattern {
    private final Pattern pattern;
    private final Class<?>[] syntaxArray;

    /**
     * 構文チェック・パターン列挙子.
     */
    public enum Pattern {
        /** チェックしない. */
        NONE,

        /** 存在すること. */
        EXIST,

        /** 存在しないこと. */
        NOT_EXIST
    }

    /**
     * コンストラクタ.
     * @param pattern 構文繰り返しパターン
     * @param syntax  構文候補配列 (トークンまたは構文のクラス)
     */
    public SyntaxPattern(Pattern pattern, Class<?> ...syntax) throws IllegalArgumentException {
        // パラメータチェック
        try {
            for (Class<?> clazz : syntax) {
                if (clazz.newInstance() instanceof AbstractToken ||
                    clazz.newInstance() instanceof AbstractSyntax) {
                } else {
                    throw new IllegalArgumentException(
                            clazz.getSimpleName()+" is not instance of Token or Syntax.");
                }
            }
        } catch(IllegalAccessException e) {
            throw new IllegalArgumentException(e);
        } catch(InstantiationException e) {
            throw new IllegalArgumentException(e);
        }

        this.pattern = pattern;
        this.syntaxArray = syntax;
    }

    /**
     * 構文パターン取得.
     * @return 構文パターン
     */
    Pattern getPattern() {
        return pattern;
    }

    /**
     * 構文候補配列取得.
     * @return 構文候補配列
     */
    Class<?>[] getSyntax() {
        return syntaxArray;
    }
}
