package com.cch_lab.android.apply.prototype.tinybasic.syntax;

/**
 * 構文木・パターン.
 * <p>
 *     構文に対応する構文木解析パターン情報を保管するクラスです。
 * </p>
 */
public class SyntaxPatterns {
    private final SyntaxPattern prevPattern;
    private final SyntaxPattern bodyPattern;
    private final SyntaxPattern nextPattern;

    /**
     * コンストラクタ.
     * @param prevPattern 前置構文パターン
     * @param bodyPattern 中置構文パターン
     * @param nextPattern 次置構文パターン
     */
    public SyntaxPatterns(SyntaxPattern prevPattern, SyntaxPattern bodyPattern, SyntaxPattern nextPattern) {
        this.prevPattern = prevPattern;
        this.bodyPattern = bodyPattern;
        this.nextPattern = nextPattern;
    }

    /**
     * 前置構文パターン配列取得.
     * @retun 前置構文パターン配列
     */
    SyntaxPattern getPrev() {
        return prevPattern;
    }

    /**
     * 中置構文パターン配列取得.
     * @retun 中置構文パターン配列
     */
    SyntaxPattern getBody() {
        return bodyPattern;
    }

    /**
     * 次置構文パターン配列取得.
     * @retun 次置構文パターン配列
     */
    SyntaxPattern getNext() {
        return nextPattern;
    }
}
