package com.cch_lab.android.apply.prototype.tinybasic.syntax;

import com.cch_lab.android.apply.prototype.tinybasic.token.AbstractToken;


/**
 * 構文木.
 * <p>
 *     構文木(構文木の入れ子構造)を表すクラスです。
 * </p>
 */
public class SyntaxTree {
    private SyntaxHolder syntaxHolder;

    /**
     * コンストラクタ.
     * @param syntaxHolder 構文・ホルダ
     */
    public SyntaxTree(SyntaxHolder syntaxHolder) {
        this.syntaxHolder = syntaxHolder;
    }

    /**
     * ノード・構文木取得.
     * @return 構文木
     */
    public SyntaxTree getNode() {
        return this;
    }

    /**
     * ノード・タイプ取得.
     * @return トークン|構文・型
     */
    public SyntaxHolder.Type getNodeType() {
        return getType(syntaxHolder);
    }

    /**
     * ノード・構文取得.
     * @return 構文（ノードが構文でない場合は、nullが返る）
     */
    public AbstractSyntax getNodeSyntax() {
        return getSyntax(syntaxHolder);
    }

    /**
     * ノード・トークン取得.
     * @return トークン （ノードがトークンでない場合は、nullが返る->常にnullが返る）
     */
    public AbstractToken getNodeToken() {
        return getToken(syntaxHolder);
    }


    /**
     * 左リーフ・構文木取得.
     * @return 構文木 （左リーフが構文木でない場合は、nullが返る）
     */
    public SyntaxTree getLeftLeaf() {
        return getSyntaxTree(syntaxHolder.getSyntax().getPrevSyntax());
    }

    /**
     * 左リーフ・タイプ取得.
     * @return トークン|構文・型
     */
    public SyntaxHolder.Type getLeftType() {
        return getType(syntaxHolder.getSyntax().getPrevSyntax());
    }

    /**
     * 左リーフ・トークン取得.
     * @return トークン （左リーフがトークンでない場合は、nullが返る）
     */
    public AbstractToken getLeftToken() {
        return getToken(syntaxHolder.getSyntax().getPrevSyntax());
    }

    /**
     * 中リーフ・構文木取得.
     * @return 構文木 （中リーフが構文木でない場合は、nullが返る）
     */
    public SyntaxTree getMiddleLeaf() {
        return getSyntaxTree(syntaxHolder.getSyntax().getBodySyntax());
    }

    /**
     * 中リーフ・タイプ取得.
     * @return トークン|構文・型
     */
    public SyntaxHolder.Type getMiddleType() {
        return getType(syntaxHolder.getSyntax().getBodySyntax());
    }

    /**
     * 中リーフ・トークン取得.
     * @return トークン （中リーフがトークンでない場合は、nullが返る）
     */
    public AbstractToken getMiddleToken() {
        return getToken(syntaxHolder.getSyntax().getBodySyntax());
    }

    /**
     * 右リーフ・構文木取得.
     * @return 構文木 （右リーフが構文木でない場合は、nullが返る）
     */
    public SyntaxTree getRightLeaf() {
        return getSyntaxTree(syntaxHolder.getSyntax().getNextSyntax());
    }

    /**
     * 右リーフ・タイプ取得.
     * @return トークン|構文・型
     */
    public SyntaxHolder.Type getRightType() {
        return getType(syntaxHolder.getSyntax().getNextSyntax());
    }

    /**
     * 右リーフ・トークン取得.
     * @return トークン （右リーフがトークンでない場合は、nullが返る）
     */
    public AbstractToken getRightToken() {
        return getToken(syntaxHolder.getSyntax().getNextSyntax());
    }

    /**
     * タイプ取得.
     * @param syntax 構文ホルダ
     * @return トークン|構文・型
     */
    SyntaxHolder.Type getType(SyntaxHolder syntax) {
        return syntax == null ? SyntaxHolder.Type.NONE : syntax.getType();
    }

    /**
     * 構文木取得.
     * @param syntax 構文ホルダ
     * @return 構文木
     */
    SyntaxTree getSyntaxTree(SyntaxHolder syntax) {
        return syntax == null
                ? null
                : syntax.getType() == SyntaxHolder.Type.SYNTAX
                    ? new SyntaxTree(syntax)
                    : null;
    }

    /**
     * 構文木取得.
     * @param syntax 構文ホルダ
     * @return 構文木
     */
    AbstractSyntax getSyntax(SyntaxHolder syntax) {
        return syntax == null
                ? null
                : syntax.getType() == SyntaxHolder.Type.SYNTAX
                    ? syntax.getSyntax()
                    : null;
    }

    /**
     * トークン取得.
     * @param syntax 構文ホルダ
     * @return トークン
     */
    AbstractToken getToken(SyntaxHolder syntax) {
        return syntax == null
                ? null
                : syntax.getType() == SyntaxHolder.Type.TOKEN
                    ? syntax.getToken()
                    : null;
    }
}
