package com.cch_lab.android.apply.prototype.tinybasic.token;

import com.cch_lab.android.apply.prototype.tinybasic.token.item.AssignmentToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.BeginBlockCommentToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.BlockCommentToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.CloseBracketToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.CloseCurlyBraceToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.CloseParenthesesToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.DivideOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.EndBlockCommentToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.EqualsOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.GreaterEqualsOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.GreaterThenOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.IntegerNumberLiteralToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.LessEqualsOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.LessThenOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.LineCommentToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.LogicalAndOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.LogicalOrOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.MiddleBlockCommentToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.MinusOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.MultiOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.NameToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.NegativeOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.NotEqualsOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.OpenBracketToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.OpenCurlyBraceToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.OpenParenthesesToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.PlusOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.ReminderOperatorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.SeparateOpratorToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.StringLiteralToken;
import com.cch_lab.android.apply.prototype.tinybasic.token.item.TerminalOperatorToken;

/**
 * トークン・マネジャ・クラス.
 */
public class TokenManager extends AbstractTokenManager {

    /**
     * デフォルト・コンストラクタ.
     */
    public TokenManager() {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void setup() {
        // 登録順序は、以下を基準とする。
        // 自己トークンパターンに、前置トークンを必要とするもの
        // 自己トークン・パターンが、他のトークンに含まれるもの

        // コメント
        register(LineCommentToken.class);
        register(BlockCommentToken.class);
        register(BeginBlockCommentToken.class);
        register(MiddleBlockCommentToken.class);
        register(EndBlockCommentToken.class);

        // 数値リテラル
        register(IntegerNumberLiteralToken.class);

        // 文字列リテラル
        register(StringLiteralToken.class);

        // 括弧
        register(OpenParenthesesToken.class);
        register(OpenBracketToken.class);
        register(OpenCurlyBraceToken.class);
        register(CloseParenthesesToken.class);
        register(CloseBracketToken.class);
        register(CloseCurlyBraceToken.class);

        // 区切演算子
        register(SeparateOpratorToken.class);

        // 終端演算子
        register(TerminalOperatorToken.class);

        // 標準演算子
        register(MultiOperatorToken.class);
        register(DivideOperatorToken.class);
        register(ReminderOperatorToken.class);
        register(PlusOperatorToken.class);
        register(MinusOperatorToken.class);

        // 比較演算子
        register(EqualsOperatorToken.class);
        register(NotEqualsOperatorToken.class);
        register(GreaterEqualsOperatorToken.class);
        register(LessEqualsOperatorToken.class);
        register(GreaterThenOperatorToken.class);
        register(LessThenOperatorToken.class);

        // 論理演算子
        register(LogicalAndOperatorToken.class);
        register(LogicalOrOperatorToken.class);
        register(NegativeOperatorToken.class);

        // 代入子
        register(AssignmentToken.class);

        // キーワードは、字句解析時には名称に含ませます。
        // 構文解析時に、変数名や関数名とキーワードを切り分けます。

        // 名称(不特定キーワード)
        register(NameToken.class);
    }
}
