package com.cch_lab.android.apply.prototype.tinybasic.token;

import java.util.regex.Pattern;


/**
 * トークン・パターン.
 * <p>
 *     トークンに対応する字句解析パターン情報を保管するクラスです。
 * </p>
 */
public class TokenPattern {
    private final boolean isNegativeCase;
    private final Pattern bodyPattern;
    private final Class<? extends AbstractToken>[] prevTokens;

    /**
     * コンストラクタ.
     * @param bodyPattern    自己トークン・パターン
     * @param isNegativeCase 前置パターン否定ケースフラグ (true:否定ケース、 false：肯定ケース)
     * @param prevTokens     前置トークン・パターン配列 (バターン不特定の場合は、空配列を指定)
     */
    public TokenPattern(
            Pattern bodyPattern,
            boolean isNegativeCase,
            Class<? extends AbstractToken>... prevTokens) {
        this.bodyPattern = bodyPattern;
        this.isNegativeCase = isNegativeCase;
        this.prevTokens = prevTokens;
    }

    /**
     * 自己トークン・パターン取得.
     * @return 自己トークン・パターン
     */
    public Pattern getBody() {
        return bodyPattern;
    }

    /**
     * 前置パターン否定ケースフラグ取得.
     * @return 前置パターン否定ケースフラグ (true:否定ケース、 false：肯定ケース)
     */
    public boolean isNegativeCase() {
        return isNegativeCase;
    }

    /**
     * 前置トークン・パターン配列取得.
     * @return 前置トークン・パターン配列 (バターン不特定の場合は、空配列となる)
     */
    public Class<? extends AbstractToken>[] getPrev() {
        return prevTokens;
    }
}
