package com.cch_lab.android.apply.prototype.tinybasic.token;

import java.lang.String;
import com.cch_lab.android.apply.prototype.tinybasic.source.SourceStream;


/**
 * トークン・ソース.
 * <p>
 *     トークンに対応するソース・ストリームと位置情報を保管します。
 * </p>
 */
public class TokenSource {
    private SourceStream sourceStream;
    private int row;
    private int start;
    private int end;

    /**
     * コンストラクタ.
     * @param sourceStream ソース・ストリーム
     * @param row   トークン・ソース行インデックス
     * @param start トークン・ソース開始位置インデックス
     * @param end   トークン・ソース終了位置インデックス
     */
    public TokenSource(SourceStream sourceStream, int row, int start, int end) {
        this.sourceStream = sourceStream;
        this.row = row;
        this.start = start;
        this.end = end;
    }

    /**
     * トークン・ソース文字列取得.
     * @return トークン・ソース文字列
     */
    public String getSource() {
        return sourceStream.getSource(row, start, end);
    }

    /**
     * ソース・ストリーム取得.
     * @return ソース・ストリーム
     */
    public SourceStream getSourceStream() {
        return sourceStream;
    }

    /**
     * トークン・ソース行インデックス取得.
     * @return トークン・ソース行インデックス
     */
    public int getRow() {
        return row;
    }

    /**
     * トークン・ソース開始位置インデックス取得.
     * @return トークン・ソース開始位置インデックス
     */
    public int getStart() {
        return start;
    }

    /**
     * トークン・ソース終了位置インデックス取得.
     * @return トークン・ソース終了位置インデックス
     */
    public int getEnd() {
        return end;
    }
}
